@@ -54,7 +54,7 @@ menu(X, Y, St) ->
 	    {?__(12,"Tighten"),tighten,
 	     ?__(13,"Move vertices towards average midpoint")},
 	    {?__(14,"Subdivide"),subdiv_fun(),
-	     {?__(15,"Subdivide all faces to give the object a smoother apperance"),[],
+	     {?__(15,"Subdivide all faces to give the object a smoother appearance"),[],
 	     ?__(53,"Subdivide all the object's faces")},[]},
 	    {?__(16,"Combine"),combine,
 	     ?__(17,"Combine multiple objects into a single object")},
@@ -642,7 +642,7 @@ combine(#st{}=St) ->
 
 separate(St) ->
     CF = fun(Items, We0) ->
-                 Empty = gb_sets:empty(),
+                 Empty = gb_sets:singleton(0),
                  [We|Wes] = wings_we:separate(We0),
                  New = [{W,Empty,sep} || W <- Wes],
                  {We,Items,New}
@@ -784,7 +784,7 @@ rename_2([], [], St) -> St.
 
 rename_qs(Objs) ->
     OldNames = [{label,Name} || #{name:=Name} <- Objs],
-    TextFields = [{text,Name,[]} || #{name:=Name} <- Objs],
+    TextFields = [{text,Name,[{width,22}]} || #{name:=Name} <- Objs],
     [{hframe,
       [{vframe,OldNames},
        {vframe,TextFields}]}].
