@@ -262,7 +262,7 @@ delete_windows(Name, W0) ->
             case get(current_gl) of
                 Obj ->
                     erase(current_gl),
-                    wxGLCanvas:setCurrent(wxwindow(geom));
+                    wings_gl:setCurrent(wxwindow(geom), ?GET(gl_context));
                 _ -> ignore
             end,
 	    erase(Obj),
@@ -451,13 +451,15 @@ grab_focus() ->
 grab_focus(Name) ->
     case is_window(Name) of
 	true  ->
-	    %%{_, [_,Where|_]} = erlang:process_info(self(), current_stacktrace),
-	    %%io:format("Grab focus: ~p~n   ~p~n",[Name, Where]),
+	    %% {_, [_,Where|_]} = erlang:process_info(self(), current_stacktrace),
 	    case get(wm_focus_grab) of
 		undefined ->
 		    update_focus(Name),
+                    %% ?dbg("Grab focus: ~p in []~n   ~p~n",[Name,Where]),
 		    put(wm_focus_grab, [Name]);
-		Stack -> put(wm_focus_grab, [Name|Stack])
+		Stack ->
+                    %% ?dbg("Grab focus: ~p in ~p~n   ~p~n",[Name,Stack,Where]),
+                    put(wm_focus_grab, [Name|Stack])
 	    end;
 	false -> ok
     end.
@@ -466,10 +468,10 @@ release_focus() ->
     case get(wm_focus_grab) of
 	undefined -> ok;
 	[_Name] ->
-            %%io:format("Release focus ~p~n",[_Name]),
+            %% ?dbg("Release focus ~p~n",[_Name]),
             erase(wm_focus_grab);
 	[_Name|Stack] ->
-            %%io:format("Release focus ~p~n",[_Name]),
+            %% ?dbg("Release focus ~p next: ~p~n",[_Name, Stack]),
             put(wm_focus_grab, Stack)
     end.
 
@@ -505,31 +507,38 @@ win_scale() ->
     #win{scale=Scale} = get_window_data(this()),
     Scale.
 
+-spec viewport() -> {integer(), integer(), integer(), integer()}.
 viewport(Name) ->
     #win{x=X,y=Y0,w=W,h=H} = get_window_data(Name),
     {_,TopH} = wings_wm:top_size(),
     Y = TopH-(Y0+H),
     {X,Y,W,H}.
 
+-spec win_size() -> {integer(), integer()}.
 win_size() ->
     {_,_,W,H} = get(wm_viewport),
     {W,H}.
 
+-spec win_size(Name::term()) -> {integer(), integer()}.
 win_size(Name) ->
     #win{w=W,h=H} = get_window_data(Name),
     {W,H}.
 
+-spec win_rect() -> {integer(), integer(), integer(), integer()}.
 win_rect() ->
     win_rect(this()).
 
+-spec win_rect(Name::term()) -> {integer(), integer(), integer(), integer()}.
 win_rect(Name) ->
     {X0,Y0,W,H} = wxWindow:getRect(wxwindow(Name)),
     {X,Y} = wxWindow:screenToClient(?GET(top_frame),{X0,Y0}),
     {X,Y,W,H}.
 
+-spec win_ul() -> {integer(), integer()}.
 win_ul() ->
     win_ul(this()).
 
+-spec win_ul(Name::term()) -> {integer(), integer()}.
 win_ul(Name) ->
     case get_window_data(Name) of
 	#win{x=X,y=Y,obj=undefined} ->
@@ -754,7 +763,7 @@ dispatch_event(#wx{event=#wxActivate{active=Active}}) ->
     end;
 dispatch_event(#wx{obj=Obj, event=#wxSize{size={W,H}}}) ->
     ?CHECK_ERROR(),
-    erase(current_gl), %% resets wxGLCanvas:setCurrent().
+    erase(current_gl), %% resets wings_gl:setCurrent().
     case W > 0 andalso H > 0 andalso not (wx2win(Obj) =:= none) of
 	true ->
 	    #win{name=Name} = Geom0 = get_window_data(Obj),
@@ -793,6 +802,9 @@ dispatch_event(#wx{obj=Obj, event=#wxFocus{type=kill_focus, win=New}}) ->
 dispatch_event(quit) ->
     foreach(fun(Name) -> send(Name, quit) end, gb_trees:keys(get(wm_windows))),
     true;
+dispatch_event(init_opengl) ->
+    foreach(fun(Name) -> send(Name, init_opengl) end, gb_trees:keys(get(wm_windows))),
+    true;
 dispatch_event({wm,WmEvent}) ->
     wm_event(WmEvent),
     true;
@@ -913,7 +925,7 @@ redraw_win({Name, #win{w=W,h=H,obj=Obj,scale=Scale}}) ->
                      case get(current_gl) of
                          Obj -> ignore;
                          _ ->
-                             wxGLCanvas:setCurrent(Obj),
+                             wings_gl:setCurrent(Obj, ?GET(gl_context)),
                              put(current_gl, Obj)
                      end,
 		     gl:viewport(0,0,round(W*Scale),round(H*Scale)),
@@ -976,11 +988,7 @@ clear_background() ->
     gl:clear(?GL_COLOR_BUFFER_BIT bor ?GL_DEPTH_BUFFER_BIT).
 
 init_opengl(Name, Canvas) ->
-    wxGLCanvas:setCurrent(Canvas),
-    gl:clear(?GL_COLOR_BUFFER_BIT bor ?GL_DEPTH_BUFFER_BIT),
-    gl:pixelStorei(?GL_UNPACK_ALIGNMENT, 1),
-    {R,G,B} = wings_pref:get_value(background_color),
-    gl:clearColor(R, G, B, 1.0),
+    wings_gl:setCurrent(Canvas, ?GET(gl_context)),
     send(Name, init_opengl).
 
 send_event(#win{z=Z}=Win, redraw) when Z < 0 ->
@@ -1040,15 +1048,15 @@ handle_event(State, Event, Stk) ->
 	    exit(shutdown);
 	exit:{crash_logged, _}=Reason ->
 	    exit(Reason);
-	exit:Exit ->
+	exit:Exit:StackTrace ->
             erase(wm_focus_grab),
 	    [#se{h=CrashHandler}] = pop_all_but_one(Stk),
-	    handle_response(CrashHandler({crash,Exit}), Event,
+	    handle_response(CrashHandler({crash,{Exit,stack,StackTrace}}), Event,
 			    default_stack(this()));
-	error:Reason ->
+	error:Reason:StackTrace ->
             erase(wm_focus_grab),
 	    [#se{h=CrashHandler}] = pop_all_but_one(Stk),
-	    handle_response(CrashHandler({crash,Reason}), Event,
+	    handle_response(CrashHandler({crash,{Reason,stack,StackTrace}}), Event,
 			    default_stack(this()))
     end.
 
@@ -1343,5 +1351,6 @@ is_valid_prop(size) ->  false;
 is_valid_prop(pos) -> false;
 is_valid_prop(internal) -> false;
 is_valid_prop(external) -> false;
+is_valid_prop(gui_win) -> false;
 is_valid_prop(_) ->  true.
 
